import { world } from '@minecraft/server';

import setIcon from "./icons"

let viewDistance = 5;

world.events.tick.subscribe(() => {
    let players = Array.from(world.getDimension("overworld").getPlayers())

    for (let i = 0; i < players.length; i++) {
        //Entity Indicator
        let lookingEntities = players[i].getEntitiesFromViewVector();
        if (lookingEntities.length != 0) {

            if (targetIsOnRange(players[i].headLocation, lookingEntities[0].location)) {

                let itemComponent = lookingEntities[0].getComponent("item")

                let tameProbability = 0;
                if (lookingEntities[0].getComponent("tameable")) {
                    tameProbability = lookingEntities[0].getComponent("tameable").probability;
                }

                let heartsToShow = lookingEntities[0].getComponent("health") != null?getHealthText(lookingEntities[0].getComponent("health").current, lookingEntities[0].getComponent("health").value):""
                let isBaby = lookingEntities[0].getComponent("is_baby") != null

                let entityName = lookingEntities[0].typeId.split(":")[1]
                let modIdentifier = itemComponent!=null?itemComponent.itemStack.typeId.split(":")[0] : lookingEntities[0].typeId.split(":")[0]

                if (itemComponent == null)
                    players[i].runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"entity.${modIdentifier == "minecraft" ? "" : modIdentifier + ":"}${entityName}.name"}, {"text":"${heartsToShow.length!=0?"\n" + heartsToShow:""}${isBaby ? "\n§7§oBaby" : ""}${tameProbability != 0 ? "\n§7§o" + tameProbability : ""}\n§1§o${modIdentifier}"}]}`)
                else {
                    players[i].runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"entity.${entityName}.name"}, {"text":" x${itemComponent.itemStack.amount}\n§1§o${modIdentifier}"}]}`)
                }

            }
        } else {

            //If not an entity, then block indicator
            let lookingBlock = players[i].getBlockFromViewVector()
            if (lookingBlock) {

                let blockName = lookingBlock.typeId.split(":")[1]
                let modIdentifier = lookingBlock.typeId.split(":")[0]

                let extraData = ``

                if (targetIsOnRange(players[i].headLocation, lookingBlock.location)) {
                    if (blockName.includes("log") && modIdentifier === "minecraft") {
                        blockName = "log." + lookingBlock.getTags()[2];
                    }
                    if (blockName == "stone")
                        blockName += ".stone"
                    else if (blockName == "seagrass")
                        blockName += ".seagrass"
                    
                    if (players[i].isSneaking) {
                        if (lookingBlock.getRedstonePower != undefined) {
                            extraData += 'Redstone Power: ' + lookingBlock.getRedstonePower()
                        }
                        extraData += `x: ${lookingBlock.x} y:${lookingBlock.y} z:${lookingBlock.z}`

                        if (lookingBlock.getComponent("sign")) {
                            extraData += "\n" + lookingBlock.getComponent("sign").text;
                        }

                        players[i].runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"tile.${modIdentifier == "minecraft" ? "" : modIdentifier + ":"}${blockName}.name"}, {"text":"\n§o§7${extraData}"}, {"text":"\n§1§o${modIdentifier}"}]}`)
                    } else {
                        players[i].runCommandAsync(`titleraw @s actionbar {"rawtext":[{"translate":"tile.${modIdentifier == "minecraft" ? "" : modIdentifier + ":"}${blockName}.name"}, {"text":"\n§1§o${modIdentifier}"}]}`)
                    }
                }
            }
        }
    }
})


function targetIsOnRange(headLocation, targetLocation) {
    return Math.abs(targetLocation.x - headLocation.x) < viewDistance && Math.abs(targetLocation.z - headLocation.z) < viewDistance && Math.abs(targetLocation.y - headLocation.y) < viewDistance
}

function getHealthText(currentHealth, maxHealth) {
    currentHealth = currentHealth - 1;
    let heartsToShow = ""
    if (maxHealth <= 20) {
        for (let i = 0; i < maxHealth; i++) {
            if (i % 2 == 0) {
                if (i < currentHealth) {
                    heartsToShow += ":::heart";
                } else if (i > currentHealth) {
                    heartsToShow += ":::hempty";
                }
            } else {
                if (i == currentHealth + 1) {
                    heartsToShow += ":::hhalf"
                }
            }
        }
        heartsToShow = setIcon(heartsToShow, 'heart', 0xe300)
        heartsToShow = setIcon(heartsToShow, 'hhalf', 0xe301)
        heartsToShow = setIcon(heartsToShow, 'hempty', 0xe302)
    } else {
        heartsToShow = "§7§oHealth: " + (currentHealth + 1) + " / " + maxHealth
    }

    return heartsToShow
}